#include "BMP280.h"
#include "IIC.h"
#include "delay/delay.h"

unsigned short dig_T1;
short dig_T2;
short dig_T3;
unsigned short dig_P1;
short dig_P2;
short dig_P3;
short dig_P4;
short dig_P5;
short dig_P6;
short dig_P7;
short dig_P8;
short dig_P9;

unsigned char bmp280_ReadByte(unsigned char addr)
{
    unsigned char temp;

    I2C_Start();
    I2C_SendData(BMP280_ADDR);
    I2C_RecvACK();
    I2C_SendData(addr);
    I2C_RecvACK();

    I2C_Start();
    I2C_SendData(BMP280_ADDR + 1);
    I2C_RecvACK();
    temp = I2C_RecvData();
    I2C_SendNAK();
    I2C_Stop();

    return temp;
}

void bmp280_WriteByte(unsigned char addr, unsigned char dat)
{
    I2C_Start();
    I2C_SendData(BMP280_ADDR);
    I2C_RecvACK();
    I2C_SendData(addr);
    I2C_RecvACK();
    I2C_SendData(dat);
    I2C_RecvACK();
    I2C_Stop();
}

long bmp280_MultipleReadThree(unsigned char addr)
{
    unsigned char msb, lsb, xlsb;
    long temp = 0;
    msb = bmp280_ReadByte(addr);
    lsb = bmp280_ReadByte(addr + 1);
    xlsb = bmp280_ReadByte(addr + 2);

    temp = (long)(((unsigned long)msb << 12)|((unsigned long)lsb << 4)|((unsigned long)xlsb >> 4));

    return temp;
}

short bmp280_MultipleReadTwo(unsigned char addr)
{
    unsigned char msb, lsb;
    short temp = 0;
    lsb = bmp280_ReadByte(addr);
    msb = bmp280_ReadByte(addr + 1);

    temp = (short)msb << 8;
    temp |= (short)lsb;

    return temp;
}

void bmp280_Init(void)
{
    unsigned char temp = 0;

    //״̬ȫ
    bmp280_WriteByte(0xe0, 0xb6);

    //ȡIDʱ֪Ϊɶˣȥ
//  temp = bmp280_ReadByte(0xd0);
//  if(temp == 0x58)
//      Uart1SendStr("bmp280 id is right...\r\n");
//  else
//      Uart1SendStr("bmp280 id is error...\r\n");

    bmp280_WriteByte(0xf4, 0xff);
    bmp280_WriteByte(0xf5, 0x00);
		delay_ms(1); 
    dig_T1 = bmp280_MultipleReadTwo(0x88);
    dig_T2 = bmp280_MultipleReadTwo(0x8A);
    dig_T3 = bmp280_MultipleReadTwo(0x8C);
    dig_P1 = bmp280_MultipleReadTwo(0x8E);
    dig_P2 = bmp280_MultipleReadTwo(0x90);
    dig_P3 = bmp280_MultipleReadTwo(0x92);
    dig_P4 = bmp280_MultipleReadTwo(0x94);
    dig_P5 = bmp280_MultipleReadTwo(0x96);
    dig_P6 = bmp280_MultipleReadTwo(0x98);
    dig_P7 = bmp280_MultipleReadTwo(0x9A);
    dig_P8 = bmp280_MultipleReadTwo(0x9C);
    dig_P9 = bmp280_MultipleReadTwo(0x9E);

//    delay_ms(200);        
}

//  dig_T1 = 27504;
//  dig_T2 = 26435;
//  dig_T3 = -1000;
//  dig_P1 = 36477;
//  dig_P2 = -10685;
//  dig_P3 = 3024;
//  dig_P4 = 2855;
//  dig_P5 = 140;
//  dig_P6 = -7;
//  dig_P7 = 15500;
//  dig_P8 = -14600;
//  dig_P9 = 6000;
//  adc_T = 519888;
//  adc_P = 415148;

float bmp280_GetPress(void)
{
    long adc_T;
    long adc_P;
    long var1, var2, t_fine, T, p;

    adc_T = bmp280_MultipleReadThree(BMP280_TEMP_ADDR);
    adc_P = bmp280_MultipleReadThree(BMP280_PRESS_ADDR);

    if(adc_P == 0)
    {
        return 0;
    }

    //Temperature
    var1 = (((float)adc_T)/16384.0-((float)dig_T1)/1024.0)*((float)dig_T2);
    var2 = ((((float)adc_T)/131072.0-((float)dig_T1)/8192.0)*(((float)adc_T)
                /131072.0-((float)dig_T1)/8192.0))*((float)dig_T3);

    t_fine = (unsigned long)(var1+var2);

    T = (var1+var2)/5120.0;

    var1 = ((float)t_fine/2.0)-64000.0;
    var2 = var1*var1*((float)dig_P6)/32768.0;
    var2 = var2 +var1*((float)dig_P5)*2.0;
    var2 = (var2/4.0)+(((float)dig_P4)*65536.0);
    var1 = (((float)dig_P3)*var1*var1/524288.0+((float)dig_P2)*var1)/524288.0;
    var1 = (1.0+var1/32768.0)*((float)dig_P1);
    p = 1048576.0-(float)adc_P;
    p = (p-(var2/4096.0))*6250.0/var1;
    var1 = ((float)dig_P9)*p*p/2147483648.0;
    var2 = p*((float)dig_P8)/32768.0;
    p = p+(var1+var2+((float)dig_P7))/16.0;

    return p;
}
float bmp280_GetTemp(void)
{
    long adc_T;

    float var1, var2, t_fine, T;

    adc_T = bmp280_MultipleReadThree(BMP280_TEMP_ADDR);


    //Temperature
    var1 = (((float)adc_T)/16384.0-((float)dig_T1)/1024.0)*((float)dig_T2);
    var2 = ((((float)adc_T)/131072.0-((float)dig_T1)/8192.0)*(((float)adc_T)
                /131072.0-((float)dig_T1)/8192.0))*((float)dig_T3);

    t_fine = (unsigned long)(var1+var2);

    return T = (var1+var2)/5120.0;
}